package be.six.reports.data;

import javax.swing.JComponent;

public abstract class Parameter {
	public static final String PATTERN = "!\\{(.*?)\\}!";
	
	protected final String type;
	protected final String description;
	
	public Parameter(String name){
		String[] data = name.split(" ");
		type = data[0].toLowerCase();
		String description = "";
		for(int i = 1 ; i<data.length;i++){
			description += data[i] + " ";
		}
		this.description=description.trim();
	}
	
	public abstract JComponent getComponent();
	public abstract Object getValue();
	public String getLabel(){
		String label = capitalize(description); 
		if(label.length()==0)
			label = capitalize(type);
		return label + ":";
	}
	
	/**
	 * Capitalizes a string. 
	 * @param s a string like "thiS StrING"
	 * @return a string like "This string"
	 */
	protected String capitalize(String s){
		if (s.length() == 0) return s;
        s = s.toLowerCase();
        s = s.substring(0, 1).toUpperCase() + s.substring(1).toLowerCase();	
        return s;
	}
	
	
	public static Parameter getParameter(String name){
		if(TypeParameter.isTypeParameter(name)) return new TypeParameter(name);
		if(SQLParameter.isSQLParameter(name)) return new SQLParameter(name);
		if(ListParameter.isListParameter(name)) return new ListParameter(name);
		throw new Error("Parameter " + name + " not recognized");
	}
}
