package be.six.reports.data;

import java.util.Calendar;
import java.util.Date;

import javax.swing.JCheckBox;
import javax.swing.JComponent;
import javax.swing.JSpinner;
import javax.swing.JTextField;
import javax.swing.SpinnerDateModel;
import javax.swing.SpinnerModel;
import javax.swing.SpinnerNumberModel;

public class TypeParameter extends Parameter{
	
	
	public static boolean isTypeParameter(String name){
		if(name == null)
			throw new IllegalArgumentException("name");
		for(ParameterType key : ParameterType.values()){
			if(name.toLowerCase().startsWith(key.name().toLowerCase()))
				return true;
		}
		return false;
	}

	JComponent component = null;
	public TypeParameter(String name){
		super(name);
		if(ParameterType.INTEGER.isType(type)){
			component = new JSpinner();
		} else if(ParameterType.DOUBLE.isType(type)){
			component = createDoubleSpinner();
		} else if(ParameterType.STRING.isType(type)){				
			component = new JTextField();
		} else if(ParameterType.BOOLEAN.isType(type)){
			component = new JCheckBox();
		} else if(ParameterType.DATE.isType(type)){
			component = createDateChooser();
		} else{
			throw new Error( name + " is recognised as a TypeParameter"); 
		}
		
	}
	
	@Override
	public Object getValue() {
		Object o = null;
		if(ParameterType.INTEGER.isType(type)){
			o = ((JSpinner)component).getModel().getValue();
		} else if(ParameterType.DOUBLE.isType(type)){
			o =((JSpinner)component).getModel().getValue();
		} else if(ParameterType.STRING.isType(type)){				
			o = ((JTextField)component).getText();
		} else if(ParameterType.BOOLEAN.isType(type)){
			o =((JCheckBox)component).isSelected();
		} else if(ParameterType.DATE.isType(type)){
			o =((JSpinner)component).getModel().getValue();
		} else{
			throw new Error("TypeParameter in incorrect state: value not found"); 
		}
		return o;
	}

	@Override
	public JComponent getComponent() {
		return component;
	}
	
	/**
	 * Creates a UI component to represent dates (and only dates)
	 * @return aa UI component to represent dates (and only dates)
	 */
	private JComponent createDateChooser(){
		Calendar calendar = Calendar.getInstance();
		
		Date initDate = calendar.getTime();
		calendar.add(Calendar.YEAR, -3000);
		Date earliestDate = calendar.getTime();
		calendar.add(Calendar.YEAR, 6000);
		Date latestDate = calendar.getTime();
		SpinnerModel dateModel = new SpinnerDateModel(initDate,
		                             earliestDate,
		                             latestDate,
		                             Calendar.YEAR);//ignored for user input
		JSpinner spinner = new JSpinner(dateModel);
		spinner.setEditor(new JSpinner.DateEditor(spinner, "dd/MM/yyyy"));
		return spinner;
	}
	
	private JComponent createDoubleSpinner(){
		SpinnerNumberModel model = new SpinnerNumberModel(0.0, -1000000000.0, 10000000000.0, 0.0001);
		JSpinner spinner =  new JSpinner(model);
		spinner.setEditor(new JSpinner.NumberEditor(spinner, "#0.0000"));
		return spinner;
	}
	
	private enum ParameterType{
		STRING,
		BOOLEAN,
		DOUBLE,
		DATE,
		INTEGER;
		
		public boolean isType(String type){
			return name().toLowerCase().equals(type);
		}
	};
}
