package be.six.reports.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.text.DecimalFormat;

import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTable;
import javax.swing.ListSelectionModel;
import javax.swing.SwingUtilities;
import javax.swing.table.DefaultTableCellRenderer;
import javax.swing.table.TableColumnModel;

import be.six.reports.data.DatabaseResult;

/**
 * The grid with a table. UI for a DatabaseResult. 
 * Doubles are formatted with "0.##"
 * @author Joren Six
 * 
 */
public class Grid extends JPanel {
	private static final long serialVersionUID = 8815982566893354329L;
	private final JScrollPane tableScrollPane;
	private final JTable table;
	private final Color defaultForegroundColor;
	private final Color defaultBackgroundColor;
	

    /**
     * Creates the grid, uses a JTable with a JDBCTableModel as model.
     */
    public Grid() {
        super();
        
        setLayout(new BorderLayout());
        table = new JTable();
        table.setPreferredScrollableViewportSize(new Dimension(520, 300));
        table.setAutoResizeMode(JTable.AUTO_RESIZE_OFF);
        table.setAutoCreateRowSorter(true);
        
        table.setRowSelectionAllowed(true);
        table.setColumnSelectionAllowed(true);
        table.setSelectionMode(ListSelectionModel.MULTIPLE_INTERVAL_SELECTION);
        tableScrollPane = new JScrollPane(table);
        table.setDefaultRenderer(Double.class, new DoubleCellRenderer());
                    
        add(tableScrollPane,BorderLayout.CENTER);   
        
        defaultForegroundColor = table.getForeground();
        defaultBackgroundColor = table.getBackground();
    }
    
    private static final DefaultTableCellRenderer internalRenderer = new DefaultTableCellRenderer();
    private static final DecimalFormat format = new DecimalFormat("0.##");
    private class DoubleCellRenderer extends DefaultTableCellRenderer{
		private static final long serialVersionUID = -426230260814391549L;
		
		@Override
		public Component getTableCellRendererComponent(JTable table,
				Object value, boolean isSelected, boolean hasFocus, int row,
				int column) {
			Double val = (Double) value;
			Component c = Grid.internalRenderer.getTableCellRendererComponent(table, value, isSelected, hasFocus, row, column);
			JLabel label =  (JLabel) c;
			label.setText(format.format(val));
			return label;
		}
    }

    
    /**
     * Sets the background color to gray while waiting
     */
    public void toggleWait(){
    	if(table.getForeground() == defaultForegroundColor && table.getBackground() == defaultBackgroundColor){
    		table.setForeground(Color.GRAY);
        	table.setBackground(Color.LIGHT_GRAY);
    	}else{
    		table.setForeground(defaultForegroundColor);
        	table.setBackground(defaultBackgroundColor);
    	}
    }
    
    /**
     * Sets a DatabaseResult in the JTable and recalculate column sizes.
     * @param result the DatabaseResult to show.
     */
    public void refreshTableContents(final DatabaseResult result){
    	
 
    	//beetje een vreemde hack die er voor zorgt dat de
		//kolommen in het begin op een juiste breedte staan
    	Runnable r = new Runnable() {
			public void run() {
				while (!table.isShowing()) {
					try {
						Thread.sleep(100);
					}
					catch (Exception e) {}
				}
				Runnable runner = new Runnable() {
					public void run() {
						JDBCTableModel model = new JDBCTableModel(result);
						table.setModel(model);
						TableColumnModel colModel = table.getColumnModel();
						int tWidth = table.getParent().getSize().width;
						int colWidth = tWidth / table.getColumnCount();
						for (int i = 0; i < colModel.getColumnCount(); i++) {
							colModel.getColumn(i).setPreferredWidth(colWidth);
						}
					}
				};
				SwingUtilities.invokeLater(runner);
			}
		};
		new Thread(r).start();
		
    }
}
