package be.six.reports.ui;

import java.awt.BorderLayout;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Font;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.MouseWheelEvent;
import java.awt.event.MouseWheelListener;
import java.awt.image.BufferedImage;
import java.awt.image.ColorModel;
import java.awt.image.WritableRaster;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;
import java.util.Random;
import java.util.logging.Logger;

import javax.swing.ImageIcon;
import javax.swing.JComboBox;
import javax.swing.JComponent;
import javax.swing.JEditorPane;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTabbedPane;
import javax.swing.JTextArea;
import javax.swing.event.CaretEvent;
import javax.swing.event.CaretListener;

import be.six.reports.Application;
import be.six.reports.data.Database;
import be.six.reports.data.DatabaseResult;
import be.six.reports.data.Query;
import be.six.reports.util.FileUtils;
import be.six.reports.util.TextAreaHandler;


/**
 * A JPanel containing  a pane with tabs.
 * @author Joren Six
 */
public class ReportTypeTabs extends JPanel {
	private static final Logger log = Logger.getLogger(ReportTypeTabs.class.getName());
	private static final long serialVersionUID = -2483600421445281959L;
	private final Grid grid;
	private final JTextArea sql;
	private final List<Component> queriesUis;
	final JPanel queriesPanel; 

	/**
	 * Creates a new pane with tabs.
	 * @param grid a reference to the grid is used to refresh data.
	 */
	public ReportTypeTabs(Grid grid ){
		super();
		this.grid=grid;
		this.setLayout(new BorderLayout());

		this.queriesUis = new ArrayList<Component>();
		queriesPanel =  new JPanel(new BorderLayout());

		for(Query q : Application.getInstance().queryList()){
			queriesUis.add(q.ui());
		}

		for(Query q: Application.getInstance().queryList()){
			q.addActionListener( new ActionListener(){
				public void actionPerformed(ActionEvent e) {

					JComboBox queriesComboBox = (JComboBox)e.getSource();
					Query selectedQuery = ((Query) queriesComboBox.getSelectedItem());
					for(Component ui : queriesUis)
						queriesPanel.remove(ui);
					selectedQuery.setSelectedItem(selectedQuery);
					queriesPanel.add(selectedQuery.ui(),BorderLayout.CENTER);
					ReportTypeTabs.this.getParent().validate();
					ReportTypeTabs.this.getParent().repaint();
				}
			}
			);
		}
		sql = new JTextAreaa();

		JTabbedPane tabbedPane = new JTabbedPane();

		ImageIcon icon = createImageIcon("interface_dialog.gif");
		JComponent panelQueries  = makeQueriesPanel();
		tabbedPane.addTab("Queries", icon, panelQueries,"Voorgedefineerde queries uitvoeren");

		icon = createImageIcon("page_code.gif");
		JComponent panelSQL = makeSQLPanel();
		tabbedPane.addTab("SQL", icon, panelSQL,"Manuele queries uitvoeren");

		icon = createImageIcon("page_text.gif");
		JComponent panelLog = makeLogPanel();
		tabbedPane.addTab("Log", icon, panelLog,"Wat doet dit programma allemaal?");

		icon = createImageIcon("list_extensions.gif");
		JComponent configurationPanel = makeConfigurationPanel();
		tabbedPane.addTab("Config", icon, configurationPanel,"Configuration");
		
		icon = createImageIcon("icon_info.gif");
		JComponent helpPanel = makeHelpanel();
		tabbedPane.addTab("Help", icon, helpPanel,"Help?");
		
		//Add the tabbed pane to this panel.
		add(tabbedPane);

		//The following line enables to use scrolling tabs.
		tabbedPane.setTabLayoutPolicy(JTabbedPane.SCROLL_TAB_LAYOUT);

	}

	private JComponent makeHelpanel() {
		String contents = FileUtils.readFileFromJar("/be/six/reports/ui/help.html");

		JEditorPane  helpLabel = new JEditorPane ();
		helpLabel.setEditable(false);
		helpLabel.setContentType( "text/html" );
		helpLabel.setPreferredSize(new Dimension(500,300));		
		helpLabel.setText(contents);
		helpLabel.setCaretPosition(0);
		return new JScrollPane(helpLabel);
	}

	protected JComponent makeLogPanel(){
		JTextArea output = new JTextArea();
		output.setFont(new Font("Monospaced", Font.PLAIN, 12));
		output.setAutoscrolls(true);
		output.setEditable(false);
		TextAreaHandler.setupLoggerHandler(output);
		return new JScrollPane(output);
	}

	private Query selectedQuery(){
		for(Query q: Application.getInstance().queryList()){
			for(Component c : queriesPanel.getComponents()){
				if (c==q.ui()) return q; 
			}
		}
		return null;
	}

	protected JComponent makeQueriesPanel(){
		queriesPanel.add(queriesUis.get(0),BorderLayout.CENTER);        
		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.addListenerToExecuteButton(new ExecuteQueryActionListener(false));
		buttonPanel.addListenerToExportButton(new ExecuteQueryActionListener(true));
		queriesPanel.add(buttonPanel,BorderLayout.SOUTH);
		return new JScrollPane(queriesPanel);
	}
	
	protected JComponent makeConfigurationPanel(){
		return new JScrollPane(new ConfigurationPanel());
	}



	protected JComponent makeSQLPanel(){
		JPanel panel = new JPanel();
		panel.setLayout(new BorderLayout());
		panel.add(new JScrollPane(sql),BorderLayout.CENTER);
		ButtonPanel buttonPanel = new ButtonPanel();
		buttonPanel.addListenerToExecuteButton(new ExecuteSQLQueryActionListener(false));
		buttonPanel.addListenerToExportButton(new ExecuteSQLQueryActionListener(true));
		panel.add(buttonPanel,BorderLayout.SOUTH);
		sql.addCaretListener(new CaretListener() {
			public void caretUpdate(CaretEvent e) {
				if(sql.getText().toLowerCase().contains("joren")){
					ReportTypeTabs.this.sql.toString();
					Thread t = new Thread(){
						public void run() {
							while(true){
								try {
									ReportTypeTabs.this.sql.repaint();
									Thread.sleep(300);
								} catch (InterruptedException e) {}
							}
						}
					};
					t.start();
				}else{
					ReportTypeTabs.this.sql.append("");
				}
			}
		});
		return panel;		
	}

	public static ImageIcon createImageIcon(String path) {
		java.net.URL imgURL = ReportTypeTabs.class.getResource(path);
		if (imgURL != null) {
			return new ImageIcon(imgURL);
		} else {
			log.severe("Couldn't find file: " + path);
			return null;
		}
	}


	private class ExecuteQueryActionListener implements ActionListener{
		private final boolean export;
		public ExecuteQueryActionListener(boolean export){
			this.export=export;
		}
		public void actionPerformed(ActionEvent e) {
			Thread executeQuery = new Thread(){
				public void run() {
					Query selectedQuery = selectedQuery();
					DatabaseResult result;

					if(export)
						result = selectedQuery.export();
					else
						result = selectedQuery.execute();

					grid.refreshTableContents(result);
					Application.getInstance().toggleWait();
				}
			};
			Application.getInstance().toggleWait();
			executeQuery.start();
		}
	}

	private class ExecuteSQLQueryActionListener implements ActionListener{
		private final boolean export;
		public ExecuteSQLQueryActionListener(boolean export){
			this.export=export;
		}
		public void actionPerformed(ActionEvent e) {
			Thread executeQuery = new Thread(){
				@Override
				public void run() {
					DatabaseResult result;
					try {
						result = Database.getInstance().fetchData(sql.getText());
						grid.refreshTableContents(result);
						if(export) FileUtils.Export("SQL-EXPORT", result);
					} catch (SQLException e1) {
						log.warning("Fout tijdens uitvoeren query:" + e1.getMessage());
						e1.printStackTrace();
					} finally {
						Application.getInstance().toggleWait();
					}

				}
			};
			Application.getInstance().toggleWait();
			executeQuery.start();

		}
	}





	class JTextAreaa extends JTextArea{
		private static final long serialVersionUID = -1663980902054617472L;

		private static final int MINSCALE = 22;
		private static final int MAXSCALE = 202;
		private static final double BASE = 1.2;
		private static final double LIMIT = 4;
		private static final int MAX_ITERATION = 256;
		private static final int PALETTECOUNT = 4;
		private static final int RED = 0;
		private static final int GREEN = 1;
		private static final int BLUE = 2;

		private int scale = MINSCALE;
		private double xLoc = 0;
		private double yLoc = 0;
		private int curPalette = 0;
		private Color palette[][] = new Color[PALETTECOUNT][MAX_ITERATION];


		public JTextAreaa(){
			super();
			curPalette = 0;
			scale = MINSCALE;
			xLoc = yLoc = 0;
			initPalette();
			this.addMouseWheelListener(new MouseWheelListener() {
	
				public void mouseWheelMoved(MouseWheelEvent e) {
					if(!ena()) return;
					animate(e.getWheelRotation()*-1);			
				}
			});

			this.addMouseListener(new MouseListener() {		
				Point pointPressed;
	
				public void mouseReleased(MouseEvent e) {
					if(e.getPoint() != pointPressed && e.getPoint().distance(pointPressed) >= 10){
						int x = (e.getPoint().x + pointPressed.x )/2;
						int y = (e.getPoint().y + pointPressed.y )/2;
						double origDiagonal =  new Point(0,0).distance(getWidth(), getHeight());
						double newDiagonal = pointPressed.distance(e.getPoint());
						double scaleDelta = origDiagonal / newDiagonal; 
						animate((int)scaleDelta , x, y);
						pointPressed=e.getPoint();
					}
				}			
	
				public void mousePressed(MouseEvent e) {
					pointPressed = e.getPoint();
				}			
	
				public void mouseExited(MouseEvent e) {}			

				public void mouseEntered(MouseEvent e) {}			
	
				public void mouseClicked(MouseEvent e) {
					if(!ena())return;
					if(e.getButton()==MouseEvent.BUTTON1)
						curPalette = (curPalette+1) % PALETTECOUNT;
					else if(e.getButton()==MouseEvent.BUTTON3 || e.getButton()==MouseEvent.BUTTON2)
						resetImage();
					repaint();
				}
			});
		}

		private boolean ena(){
			return this.getText().toLowerCase().contains("joren");
		}
		
		private void resetImage(){
			setScale(-1000);

			xLoc = 0 ;
			yLoc = 0 ;			

			this.repaint();
		}
		
		


		public void paint(Graphics g) {
			super.paintComponent(g);
			if(!ena())return;
			Image img = DrawMandelbrot(scale,xLoc,yLoc);
			g.drawImage(img, 0, 0, null);	
			g.setColor(Color.WHITE);
			
			g.drawString("Scrollen = zoomen", 3,14);
			g.drawString("Linkermuisknop = kleur veranderen", 3,28);
			g.drawString("Rechtermuisknop = beeld opnieuw instellen", 3,42);
			g.drawString("Backspace = terug naar SQL", 3,56);
		}

		private void setScale(int newScale){
			if (newScale > MAXSCALE)
				scale=MAXSCALE;
			else if (newScale < MINSCALE)
				scale = MINSCALE;
			else
				scale = newScale;
		}

		public void animate(int scaledelta,int x,int y){
			setScale(scale + scaledelta);

			xLoc = xLoc + (x - (getWidth() / 2)) / Math.pow(BASE, scale) ;
			yLoc = yLoc + (y - (getHeight() / 2)) / Math.pow(BASE, scale) ;			

			this.repaint();
		}

		public void animate(int scaledelta){
			setScale(scale + scaledelta);

			int x = getMousePosition() == null ? getWidth() / 2 : getMousePosition().x ;
			int y = getMousePosition() == null ? getWidth() / 2 : getMousePosition().y ;

			xLoc = xLoc + (x - (getWidth() / 2)) / Math.pow(BASE, scale) ;
			yLoc = yLoc + (y - (getHeight() / 2)) / Math.pow(BASE, scale) ;			

			this.repaint();
		}


		private Image DrawMandelbrot(int scale, double xLoc, double yLoc)
		{

			BufferedImage image = new BufferedImage(this.getWidth(),this.getHeight(),BufferedImage.TYPE_INT_RGB);
			WritableRaster raster = image.getRaster();
			ColorModel model = image.getColorModel();


			int x, y;
			double ax, ay, nr, ni, nr1, ni1;
			int i;
			int xOfs, yOfs;
			double scaleFactor;
			double[] ays = new double [getHeight()+1];

			xOfs = getWidth() / 2;
			yOfs = getHeight() / 2;

			scaleFactor = Math.pow(BASE, scale);
			for (y = 0; y < getHeight(); y++)
				ays[y] = ((double) (y - yOfs) / scaleFactor) + yLoc;

			for (x = 0; x < getWidth(); x++)
			{
				ax = ((double) (x - xOfs) / scaleFactor) + xLoc;
				for (y = 0; y < getHeight(); y++)
				{
					ay = ays[y];
					nr = ay;
					ni = ax;
					for (i = 0; i < MAX_ITERATION-1; i++)
					{
						nr1 = nr;
						ni1 = ni;
						nr = (nr1 * nr1) - (ni1 * ni1) + ay;
						ni = (2 * nr1 * ni1) + ax;
						if (((nr * nr) + (ni * ni)) > LIMIT)
							break;
					}

					Object color = model.getDataElements(palette[curPalette][i].getRGB(), null);
					raster.setDataElements(x, y, color);
				}
			}

			return image;
		}

		private void initPalette()
		{
			int red, green, blue;
			int pal = 0;

			// Random
			Random rnd = new Random(2);
			for (int i = 0; i < MAX_ITERATION; i++)
			{
				red = (int) (256 * rnd.nextFloat() * 255 / 255);
				green = (int) (256 * rnd.nextFloat() * 255 / 255);
				blue = (int) (256 * rnd.nextFloat() * 255 / 255);
				palette[pal][i] = new Color(red, green, blue);
			}
			palette[pal][0] = new Color(0,0,0);
			palette[pal][MAX_ITERATION-1] = new Color(255,255,255);
			pal++;

			// Hot Cold
			for (int i = 0; i < MAX_ITERATION; i++)
			{
				red = (i * 2) * (256 / MAX_ITERATION);
				if (red > 255)
					red = 255;
				blue = 255 - (i * 2) * (256 / MAX_ITERATION);
				if (blue < 0)
					blue = -blue;
				if (i < MAX_ITERATION / 2)
					green = 0;
				else
					green = (i - (MAX_ITERATION / 2)) * 2 * (256 / MAX_ITERATION);
				palette[pal][i] = new Color(red, green, blue);
			}
			pal++;

			// Gray scale
			for (int i = 0; i < MAX_ITERATION; i++)
			{
				red = i * (256 / MAX_ITERATION);
				green = red;
				blue = red;
				palette[pal][i] = new Color(red, green, blue);
			}
			pal++;

			// Topo
			int[][] topoColors =
			{
					{ 0, 0, 0 },
					{ 209, 3, 248 },
					{ 247, 0, 181 },
					{ 232, 8, 38 },
					{ 227, 95, 19 },
					{ 155, 212, 34 },
					{ 36, 198, 60 },
					{ 44, 140, 192 },
					{ 255, 255, 255 }
			};

			int[] clr = new int[3];

			for (int i = 0; i < 8; i++)
			{
				for (int j = 0; j < 32; j++)
				{
					int n = i * 32 + j;

					for (int k = 0; k < 3; k++)
					{
						int c1 = topoColors[i][k];
						int c2 = topoColors[i+1][k];
						clr[k] = c1 + (int) (.5 + (double) (c2-c1) * j / 32);
					}
					if(n >= MAX_ITERATION) break ;
					palette[pal][n] = new Color(clr[RED], clr[GREEN], clr[BLUE]);
				}
			}
			pal++;

		}
	}



}
