package be.six.reports.util;

import java.util.Vector;

import be.six.reports.data.DatabaseResult;

/**
 * A Generic ListItem. Is used in a Vector so list items can be shown in a user interface element.
 * Defines an sorting order on the string representation of the value of  a ListItem. 
 * @author Joren Six
  */
public class ListItem  implements Comparable<ListItem>{
	private final Object key;
	private final String value;
	
	/**
	 * Creates a new list item.
	 * @param key The key of the item.
	 * @param value The value of the item. The value should have a string representation.
	 */
	public ListItem(Object key, String value){
		this.key=key;
		this.value=value;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Object#toString()
	 */
	public String toString(){
		return this.value;
	}
	
	/**
	 * @return The key of the item.
	 */
	public Object getKey(){
		return this.key;
	}
	
	/**
	 * @return The value, name of the item.
	 */
	public String getValue(){
		return this.value;
	}
	
	/**
	 * Converts a DatabaseResult to a list of ListItems. 
	 * Uses the object in the first column as key, the object in the second column as value. 
	 * @param result the result to convert.
	 * @return A Vector of ListItems.
	 */
	public static Vector<ListItem> convertDatabaseResult(DatabaseResult result){
		Vector<ListItem> items=new Vector<ListItem>();
		for(int i= 0 ;i< result.getRowCount() ;i++){
			items.add(new ListItem(result.getValueAt(i,0),result.getValueAt(i,1).toString()));
		}
		return items;
	}
	
	/* (non-Javadoc)
	 * @see java.lang.Comparable#compareTo(java.lang.Object)
	 */
	public int compareTo(ListItem o) {
		return this.value.compareTo(o.value);
	}
	
	
}
