package be.six.reports.util;

/**
 * Utility class to measure and represent time intervals
 * @author Joren Six
 
 */
public class StopWatch {
	private long startTime = 0;
    private long stopTime = 0;
    private boolean running = false;

    
    /**
     * Starts the timer.
     */
    public void start() {
        this.startTime = System.currentTimeMillis();
        this.running = true;
    }

    
    /**
     * Stops the timer.
     */
    public void stop() {
        this.stopTime = System.currentTimeMillis();
        this.running = false;
    }

     /**
     * @return elapsed time in milliseconds between start and stop. 
     */
    public long getElapsedTime() {
        long elapsed;
        if (running) {
             elapsed = (System.currentTimeMillis() - startTime);
        }
        else {
            elapsed = (stopTime - startTime);
        }
        return elapsed;
    }
    
    /**
     * @return elapsed time in seconds between start and stop.
     */
    public long getElapsedTimeSecs() {
        long elapsed;
        if (running) {
            elapsed = ((System.currentTimeMillis() - startTime) / 1000);
        }
        else {
            elapsed = ((stopTime - startTime) / 1000);
        }
        return elapsed;
    }
    
    /**
     * @return a string representing the elapsed time. E.g. 3s 157ms or 0s 157ms
     */
    public String getElapsedString(){
    	long sec = getElapsedTimeSecs();
    	long ms = getElapsedTime();
    	ms = ms - 1000 * sec;
    	return sec + "s " + ms +"ms";
    }

}
